const C3 = globalThis.C3;

C3.Plugins.yabbi.Instance = class yabbiInstance extends (
  globalThis.ISDKInstanceBase
) {
  constructor() {
    super();

    this.publisherId = "";

    function _debugLog(method) {
      console.log(
        `[sspnet-construct3] Call to ${method} on not supported platform. To test advertising, install your application on the Android/iOS device.`
      );
    }

    // Dummy methods
    const _dummyInitialize = (publisherId) => {
      _debugLog("Initialize");
    };

    const _dummySetUserConsent = (hasConsent) => {
      _debugLog("SetUserConsent");
    };

    const _dummySetCustomParams = (key, value) => {
      _debugLog("SetCustomParams");
    };

    const _dummyEnableDebug = (enabled) => {
      _debugLog("EnableDebug");
    };

    const _dummyHasUserConsent = () => {
      _debugLog("HasUserConsent");
      return false;
    };

    const _dummyLoadInterstitialAd = (placementName) => {
      _debugLog("LoadInterstitialAd");
    };
    const _dummyShowInterstitialAd = (placementName) => {
      _debugLog("ShowInterstitialAd");
    };
    const _dummyDestroyInterstitialAd = (placementName) => {
      _debugLog("DestroyInterstitialAd");
    };
    const _dummyIsInterstitialAdReady = (placementName) => {
      _debugLog("IsInterstitialAdReady");
      return false;
    };

    const _dummyLoadRewardedAd = (placementName) => {
      _debugLog("LoadRewardedAd");
    };
    const _dummyShowRewardedAd = (placementName) => {
      _debugLog("ShowRewardedAd");
    };
    const _dummyDestroyRewardedAd = (placementName) => {
      _debugLog("DestroyRewardedAd");
    };
    const _dummyIsRewardedAdReady = (placementName) => {
      _debugLog("IsRewardedAdReady");
      return false;
    };

    const _dummyLoadBannerAd = (placementName) => {
      _debugLog("LoadBannerAd");
    };
    const _dummyShowBannerAd = (placementName) => {
      _debugLog("ShowBannerAd");
    };
    const _dummyDestroyBannerAd = (placementName) => {
      _debugLog("DestroyBannerAd");
    };
    const _dummyIsBannerAdReady = (placementName) => {
      _debugLog("IsBannerAdReady");
      return false;
    };
    const _dummySetBannerCustomSettings = (showCloseButton) => {
      _debugLog("SetBannerCustomSettings");
    };

    if (typeof cordova == "undefined") {
      this._initialize = _dummyInitialize;
      this._setUserConsent = _dummySetUserConsent;
      this._hasUserConsent = _dummyHasUserConsent;
      this._enableDebug = _dummyEnableDebug;
      this._setCustomParams = _dummySetCustomParams;

      this._loadInterstitialAd = _dummyLoadInterstitialAd;
      this._showInterstitialAd = _dummyShowInterstitialAd;
      this._destroyInterstitialAd = _dummyDestroyInterstitialAd;
      this._isInterstitialAdReady = _dummyIsInterstitialAdReady;

      this._loadRewardedAd = _dummyLoadRewardedAd;
      this._showRewardedAd = _dummyShowRewardedAd;
      this._destroyRewardedAd = _dummyDestroyRewardedAd;
      this._isRewardedAdReady = _dummyIsRewardedAdReady;

      this._loadBannerAd = _dummyLoadBannerAd;
      this._showBannerAd = _dummyShowBannerAd;
      this._destroyBannerAd = _dummyDestroyBannerAd;
      this._isBannerAdReady = _dummyIsBannerAdReady;
      this._setBannerCustomSettings = _dummySetBannerCustomSettings;

      return;
    }

    this.Conditions = C3.Plugins.yabbi.Cnds;
    this.Yabbi = cordova.require("cordova.plugin.yabbi.Yabbi");

    const properties = this._getInitProperties();

    if (properties) {
      this.publisherId = properties[0];
    }

    if (typeof this.Yabbi == "undefined") {
      return;
    }

    const self = this;

    // Methods

    // Settings
    const _initialize = (publisherId) => {
      var isInitialized = self.Yabbi.isInitialized;

      if (!isInitialized) {
        self.publisherId = publisherId;

        self.Yabbi.setCustomParams(
          "PUBLISHER_URL",
          "https://yabbi.me/publishers"
        );
        self.Yabbi.setCustomParams("CABINET", "Yabbi");
        self.Yabbi.setCustomParams("PLATFORM", "construct3");

        self.Yabbi.initialize(self.publisherId, function (error) {
          if (error) {
            console.error(error);
            self._trigger(self.Conditions.OnInitializedFailed);
          } else {
            self._trigger(self.Conditions.OnInitializedSuccess);
          }
        });
      }
    };

    // Privacy Methods

    const _setUserConsent = (hasConsent) => {
      self.Yabbi.setUserConsent(hasConsent);
    };

    const _setCustomParams = (key, value) => {
      self.Yabbi.setCustomParams(key, value);
    };

    const _enableDebug = (enabled) => {
      self.Yabbi.enableDebug(enabled);
    };

    const _hasUserConsent = () => {
      return self.Yabbi.hasUserConsent;
    };

    // Ads Methods
    const _loadInterstitialAd = (placementName) => {
      self.Yabbi.loadAd(self.Yabbi.INTERSTITIAL, placementName);
    };
    const _showInterstitialAd = (placementName) => {
      self.Yabbi.showAd(self.Yabbi.INTERSTITIAL, placementName);
    };
    const _destroyInterstitialAd = (placementName) => {
      self.Yabbi.destroyAd(self.Yabbi.INTERSTITIAL, placementName);
    };
    const _isInterstitialAdReady = (placementName) => {
      return self.Yabbi.isAdLoaded(self.Yabbi.INTERSTITIAL, placementName);
    };

    const _loadRewardedAd = (placementName) => {
      self.Yabbi.loadAd(self.Yabbi.REWARDED, placementName);
    };
    const _showRewardedAd = (placementName) => {
      self.Yabbi.showAd(self.Yabbi.REWARDED, placementName);
    };
    const _destroyRewardedAd = (placementName) => {
      self.Yabbi.destroyAd(self.Yabbi.REWARDED, placementName);
    };
    const _isRewardedAdReady = (placementName) => {
      return self.Yabbi.isAdLoaded(self.Yabbi.REWARDED, placementName);
    };

    const _loadBannerAd = (placementName) => {
      self.Yabbi.loadAd(self.Yabbi.BANNER, placementName);
    };
    const _showBannerAd = (placementName) => {
      self.Yabbi.showAd(self.Yabbi.BANNER, placementName);
    };
    const _destroyBannerAd = (placementName) => {
      self.Yabbi.destroyAd(self.Yabbi.BANNER, placementName);
    };
    const _isBannerAdReady = (placementName) => {
      return self.Yabbi.isAdLoaded(self.Yabbi.BANNER, placementName);
    };
    const _setBannerCustomSettings = (showCloseButton) => {
      self.Yabbi.setBannerCustomSettings(showCloseButton);
    };

    // Register Methods
    this._initialize = _initialize;
    this._setUserConsent = _setUserConsent;
    this._hasUserConsent = _hasUserConsent;
    this._enableDebug = _enableDebug;
    this._setCustomParams = _setCustomParams;

    this._loadInterstitialAd = _loadInterstitialAd;
    this._showInterstitialAd = _showInterstitialAd;
    this._destroyInterstitialAd = _destroyInterstitialAd;
    this._isInterstitialAdReady = _isInterstitialAdReady;

    this._loadRewardedAd = _loadRewardedAd;
    this._showRewardedAd = _showRewardedAd;
    this._destroyRewardedAd = _destroyRewardedAd;
    this._isRewardedAdReady = _isRewardedAdReady;

    this._loadBannerAd = _loadBannerAd;
    this._showBannerAd = _showBannerAd;
    this._destroyBannerAd = _destroyBannerAd;
    this._isBannerAdReady = _isBannerAdReady;
    this._setBannerCustomSettings = _setBannerCustomSettings;

    // Register Trigger Events

    // Interstitial Ad
    globalThis.addEventListener("onInterstitialLoaded", (adInfo) => {
      self._trigger(self.Conditions.OnInterstitialLoaded);
    });
    globalThis.addEventListener("onInterstitialLoadFailed", (adInfo) => {
      self._trigger(self.Conditions.OnInterstitialLoadFailed);
    });
    globalThis.addEventListener("onInterstitialShown", (adInfo) => {
      self._trigger(self.Conditions.OnInterstitialShown);
    });
    globalThis.addEventListener("onInterstitialShowFailed", (adInfo) => {
      self._trigger(self.Conditions.OnInterstitialShowFailed);
    });
    globalThis.addEventListener("onInterstitialClosed", (adInfo) => {
      self._trigger(self.Conditions.OnInterstitialClosed);
    });

    globalThis.addEventListener("onBannerLoaded", function (adInfo) {
      self._trigger(self.Conditions.OnBannerLoaded);
    });
    globalThis.addEventListener("onBannerLoadFailed", function (adInfo) {
      self._trigger(self.Conditions.OnBannerLoadFailed);
    });
    globalThis.addEventListener("onBannerShown", function (adInfo) {
      self._trigger(self.Conditions.OnBannerShown);
    });
    globalThis.addEventListener("onBannerShowFailed", function (adInfo) {
      self._trigger(self.Conditions.OnBannerShowFailed);
    });
    globalThis.addEventListener("onBannerClosed", function (adInfo) {
      self._trigger(self.Conditions.OnBannerClosed);
    });
    globalThis.addEventListener("onBannerImpression", function (adInfo) {
      self._trigger(self.Conditions.OnBannerImpression);
    });

    // Rewarded Ad
    globalThis.addEventListener("onRewardedLoaded", (adInfo) => {
      self._trigger(self.Conditions.OnRewardedLoaded);
    });
    globalThis.addEventListener("onRewardedLoadFailed", (adInfo) => {
      self._trigger(self.Conditions.OnRewardedLoadFailed);
    });
    globalThis.addEventListener("onRewardedShown", (adInfo) => {
      self._trigger(self.Conditions.OnRewardedShown);
    });
    globalThis.addEventListener("onRewardedShowFailed", (adInfo) => {
      self._trigger(self.Conditions.OnRewardedShowFailed);
    });
    globalThis.addEventListener("onRewardedClosed", (adInfo) => {
      self._trigger(self.Conditions.OnRewardedClosed);
    });
    globalThis.addEventListener("onRewardedVideoStarted", function (adInfo) {
      self._trigger(self.Conditions.OnRewardedVideoStarted);
    });
    globalThis.addEventListener("onRewardedVideoCompleted", function (adInfo) {
      self._trigger(self.Conditions.OnRewardedVideoCompleted);
    });
    globalThis.addEventListener("onUserRewarded", function (adInfo) {
      self._trigger(self.Conditions.OnUserRewarded);
    });
  }

  _getDebuggerProperties() {
    return [
      {
        title: "Yabbi",
        properties: [],
      },
    ];
  }
};
